@@ -9,64 +9,92 @@ part of dart.io;
  */
 class FileMode {
   /// The mode for opening a file only for reading.
-  static const READ = const FileMode._internal(0);
+  static const read = const FileMode._internal(0);
+  @Deprecated("Use read instead")
+  static const READ = read;
 
   /// Mode for opening a file for reading and writing. The file is
   /// overwritten if it already exists. The file is created if it does not
   /// already exist.
-  static const WRITE = const FileMode._internal(1);
+  static const write = const FileMode._internal(1);
+  @Deprecated("Use write instead")
+  static const WRITE = write;
 
   /// Mode for opening a file for reading and writing to the
   /// end of it. The file is created if it does not already exist.
-  static const APPEND = const FileMode._internal(2);
+  static const append = const FileMode._internal(2);
+  @Deprecated("Use append instead")
+  static const APPEND = append;
 
   /// Mode for opening a file for writing *only*. The file is
   /// overwritten if it already exists. The file is created if it does not
   /// already exist.
-  static const WRITE_ONLY = const FileMode._internal(3);
+  static const writeOnly = const FileMode._internal(3);
+  @Deprecated("Use writeOnly instead")
+  static const WRITE_ONLY = writeOnly;
 
   /// Mode for opening a file for writing *only* to the
   /// end of it. The file is created if it does not already exist.
-  static const WRITE_ONLY_APPEND = const FileMode._internal(4);
+  static const writeOnlyAppend = const FileMode._internal(4);
+  @Deprecated("Use writeOnlyAppend instead")
+  static const WRITE_ONLY_APPEND = writeOnlyAppend;
+
   final int _mode;
 
   const FileMode._internal(this._mode);
 }
 
 /// The mode for opening a file only for reading.
-const READ = FileMode.READ;
+@Deprecated("Use FileMode.read instead")
+const READ = FileMode.read;
 
 /// The mode for opening a file for reading and writing. The file is
 /// overwritten if it already exists. The file is created if it does not
 /// already exist.
-const WRITE = FileMode.WRITE;
+@Deprecated("Use FileMode.write instead")
+const WRITE = FileMode.write;
 
 /// The mode for opening a file for reading and writing to the
 /// end of it. The file is created if it does not already exist.
-const APPEND = FileMode.APPEND;
+@Deprecated("Use FileMode.append instead")
+const APPEND = FileMode.append;
 
 /// Mode for opening a file for writing *only*. The file is
 /// overwritten if it already exists. The file is created if it does not
 /// already exist.
-const WRITE_ONLY = FileMode.WRITE_ONLY;
+@Deprecated("Use FileMode.writeOnly instead")
+const WRITE_ONLY = FileMode.writeOnly;
 
 /// Mode for opening a file for writing *only* to the
 /// end of it. The file is created if it does not already exist.
-const WRITE_ONLY_APPEND = FileMode.WRITE_ONLY_APPEND;
+@Deprecated("Use FileMode.writeOnlyAppend instead")
+const WRITE_ONLY_APPEND = FileMode.writeOnlyAppend;
 
 /// Type of lock when requesting a lock on a file.
-enum FileLock {
+class FileLock {
   /// Shared file lock.
-  SHARED,
+  static const shared = const FileLock._internal(1);
+  @Deprecated("Use shared instead")
+  static const SHARED = shared;
 
   /// Exclusive file lock.
-  EXCLUSIVE,
+  static const exclusive = const FileLock._internal(2);
+  @Deprecated("Use exclusive instead")
+  static const EXCLUSIVE = exclusive;
 
   /// Blocking shared file lock.
-  BLOCKING_SHARED,
+  static const blockingShared = const FileLock._internal(3);
+  @Deprecated("Use blockingShared instead")
+  static const BLOCKING_SHARED = blockingShared;
 
   /// Blocking exclusive file lock.
-  BLOCKING_EXCLUSIVE,
+  static const blockingExclusive = const FileLock._internal(4);
+  @Deprecated("Use blockingExclusive instead")
+  static const BLOCKING_EXCLUSIVE = blockingExclusive;
+
+  final int _type;
+
+  const FileLock._internal(this._type);
 }
 
 /**
@@ -136,7 +164,7 @@ enum FileLock {
  *       Stream<List<int>> inputStream = file.openRead();
  *
  *       inputStream
- *         .transform(UTF8.decoder)       // Decode bytes to UTF8.
+ *         .transform(utf8.decoder)       // Decode bytes to UTF-8.
  *         .transform(new LineSplitter()) // Convert stream to individual lines.
  *         .listen((String line) {        // Process results.
  *             print('$line: ${line.length} bytes');
@@ -197,18 +225,14 @@ enum FileLock {
  *
  * ## Other resources
  *
- * * [Dart by Example](https://www.dartlang.org/dart-by-example/#files-directories-and-symlinks)
- *   provides additional task-oriented code samples that show how to use
- *   various API from the Directory class and the related [File] class.
+ * * The [Files and directories](https://dart.dev/guides/libraries/library-tour#files-and-directories)
+ *   section of the library tour.
  *
- * * [I/O for Command-Line
- *   Apps](https://www.dartlang.org/docs/dart-up-and-running/ch03.html#dartio---io-for-command-line-apps)
- *   a section from _A Tour of the Dart Libraries_ covers files and directories.
- *
- * * [Write Command-Line Apps](https://www.dartlang.org/docs/tutorials/cmdline/),
+ * * [Write Command-Line Apps](https://dart.dev/tutorials/server/cmdline),
  *   a tutorial about writing command-line apps, includes information about
  *   files and directories.
  */
+@pragma("vm:entry-point")
 abstract class File implements FileSystemEntity {
   /**
    * Creates a [File] object.
@@ -219,7 +243,14 @@ abstract class File implements FileSystemEntity {
    * If [path] is an absolute path, it will be immune to changes to the
    * current working directory.
    */
-  factory File(String path) => new _File(path);
+  @pragma("vm:entry-point")
+  factory File(String path) {
+    final IOOverrides? overrides = IOOverrides.current;
+    if (overrides == null) {
+      return new _File(path);
+    }
+    return overrides.createFile(path);
+  }
 
   /**
    * Create a File object from a URI.
@@ -228,6 +259,16 @@ abstract class File implements FileSystemEntity {
    */
   factory File.fromUri(Uri uri) => new File(uri.toFilePath());
 
+  /**
+   * Creates a File object from a raw path, that is, a sequence of bytes
+   * as represented by the OS.
+   */
+  @pragma("vm:entry-point")
+  factory File.fromRawPath(Uint8List rawPath) {
+    // TODO(bkonyi): Handle overrides.
+    return new _File.fromRawPath(rawPath);
+  }
+
   /**
    * Create the file. Returns a `Future<File>` that completes with
    * the file when it has been created.
@@ -322,8 +363,8 @@ abstract class File implements FileSystemEntity {
 /**
  * Get the last-accessed time of the file.
  *
- * Returns the date and time when the file was last accessed, if the
- * information is available.
+ * Returns a `Future<DateTime>` that completes with the date and time when the
+ * file was last accessed, if the information is available.
  *
  * Throws a [FileSystemException] if the operation fails.
  */
@@ -343,6 +384,8 @@ abstract class File implements FileSystemEntity {
   /**
    * Modifies the time the file was last accessed.
    *
+   * Returns a [Future] that completes once the operation has completed.
+   *
    * Throws a [FileSystemException] if the time cannot be set.
    */
   Future setLastAccessed(DateTime time);
@@ -357,8 +400,8 @@ abstract class File implements FileSystemEntity {
 /**
  * Get the last-modified time of the file.
  *
- * Returns the date and time when the file was last modified, if the
- * information is available.
+ * Returns a `Future<DateTime>` that completes with the date and time when the
+ * file was last modified, if the information is available.
  *
  * Throws a [FileSystemException] if the operation fails.
  */
@@ -378,6 +421,8 @@ abstract class File implements FileSystemEntity {
   /**
    * Modifies the time the file was last modified.
    *
+   * Returns a [Future] that completes once the operation has completed.
+   *
    * Throws a [FileSystemException] if the time cannot be set.
    */
   Future setLastModified(DateTime time);
@@ -397,16 +442,16 @@ abstract class File implements FileSystemEntity {
    *
    * Files can be opened in three modes:
    *
-   * [FileMode.READ]: open the file for reading.
+   * [FileMode.read]: open the file for reading.
    *
-   * [FileMode.WRITE]: open the file for both reading and writing and
+   * [FileMode.write]: open the file for both reading and writing and
    * truncate the file to length zero. If the file does not exist the
    * file is created.
    *
-   * [FileMode.APPEND]: same as [FileMode.WRITE] except that the file is
+   * [FileMode.append]: same as [FileMode.write] except that the file is
    * not truncated.
    */
-  Future<RandomAccessFile> open({FileMode mode: FileMode.READ});
+  Future<RandomAccessFile> open({FileMode mode: FileMode.read});
 
   /**
    * Synchronously open the file for random access operations. The
@@ -418,7 +463,7 @@ abstract class File implements FileSystemEntity {
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  RandomAccessFile openSync({FileMode mode: FileMode.READ});
+  RandomAccessFile openSync({FileMode mode: FileMode.read});
 
   /**
    * Create a new independent [Stream] for the contents of this file.
@@ -433,7 +478,7 @@ abstract class File implements FileSystemEntity {
    * must be read to completion or the subscription on the stream must
    * be cancelled.
    */
-  Stream<List<int>> openRead([int start, int end]);
+  Stream<List<int>> openRead([int? start, int? end]);
 
   /**
    * Creates a new independent [IOSink] for the file. The
@@ -442,8 +487,8 @@ abstract class File implements FileSystemEntity {
    *
    * An [IOSink] for a file can be opened in two modes:
    *
-   * * [FileMode.WRITE]: truncates the file to length zero.
-   * * [FileMode.APPEND]: sets the initial write position to the end
+   * * [FileMode.write]: truncates the file to length zero.
+   * * [FileMode.append]: sets the initial write position to the end
    *   of the file.
    *
    *  When writing strings through the returned [IOSink] the encoding
@@ -451,21 +496,21 @@ abstract class File implements FileSystemEntity {
    *  has an `encoding` property which can be changed after the
    *  [IOSink] has been created.
    */
-  IOSink openWrite({FileMode mode: FileMode.WRITE, Encoding encoding: UTF8});
+  IOSink openWrite({FileMode mode: FileMode.write, Encoding encoding: utf8});
 
   /**
    * Read the entire file contents as a list of bytes. Returns a
-   * `Future<List<int>>` that completes with the list of bytes that
+   * `Future<Uint8List>` that completes with the list of bytes that
    * is the contents of the file.
    */
-  Future<List<int>> readAsBytes();
+  Future<Uint8List> readAsBytes();
 
   /**
    * Synchronously read the entire file contents as a list of bytes.
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  List<int> readAsBytesSync();
+  Uint8List readAsBytesSync();
 
   /**
    * Read the entire file contents as a string using the given
@@ -474,7 +519,7 @@ abstract class File implements FileSystemEntity {
    * Returns a `Future<String>` that completes with the string once
    * the file contents has been read.
    */
-  Future<String> readAsString({Encoding encoding: UTF8});
+  Future<String> readAsString({Encoding encoding: utf8});
 
   /**
    * Synchronously read the entire file contents as a string using the
@@ -482,7 +527,7 @@ abstract class File implements FileSystemEntity {
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  String readAsStringSync({Encoding encoding: UTF8});
+  String readAsStringSync({Encoding encoding: utf8});
 
   /**
    * Read the entire file contents as lines of text using the given
@@ -491,7 +536,7 @@ abstract class File implements FileSystemEntity {
    * Returns a `Future<List<String>>` that completes with the lines
    * once the file contents has been read.
    */
-  Future<List<String>> readAsLines({Encoding encoding: UTF8});
+  Future<List<String>> readAsLines({Encoding encoding: utf8});
 
   /**
    * Synchronously read the entire file contents as lines of text
@@ -499,7 +544,7 @@ abstract class File implements FileSystemEntity {
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  List<String> readAsLinesSync({Encoding encoding: UTF8});
+  List<String> readAsLinesSync({Encoding encoding: utf8});
 
   /**
    * Write a list of bytes to a file.
@@ -510,13 +555,13 @@ abstract class File implements FileSystemEntity {
    *
    * By default [writeAsBytes] creates the file for writing and truncates the
    * file if it already exists. In order to append the bytes to an existing
-   * file, pass [FileMode.APPEND] as the optional mode parameter.
+   * file, pass [FileMode.append] as the optional mode parameter.
    *
    * If the argument [flush] is set to `true`, the data written will be
    * flushed to the file system before the returned future completes.
    */
   Future<File> writeAsBytes(List<int> bytes,
-      {FileMode mode: FileMode.WRITE, bool flush: false});
+      {FileMode mode: FileMode.write, bool flush: false});
 
   /**
    * Synchronously write a list of bytes to a file.
@@ -525,7 +570,7 @@ abstract class File implements FileSystemEntity {
    *
    * By default [writeAsBytesSync] creates the file for writing and truncates
    * the file if it already exists. In order to append the bytes to an existing
-   * file, pass [FileMode.APPEND] as the optional mode parameter.
+   * file, pass [FileMode.append] as the optional mode parameter.
    *
    * If the [flush] argument is set to `true` data written will be
    * flushed to the file system before returning.
@@ -533,7 +578,7 @@ abstract class File implements FileSystemEntity {
    * Throws a [FileSystemException] if the operation fails.
    */
   void writeAsBytesSync(List<int> bytes,
-      {FileMode mode: FileMode.WRITE, bool flush: false});
+      {FileMode mode: FileMode.write, bool flush: false});
 
   /**
    * Write a string to a file.
@@ -544,15 +589,15 @@ abstract class File implements FileSystemEntity {
    *
    * By default [writeAsString] creates the file for writing and truncates the
    * file if it already exists. In order to append the bytes to an existing
-   * file, pass [FileMode.APPEND] as the optional mode parameter.
+   * file, pass [FileMode.append] as the optional mode parameter.
    *
    * If the argument [flush] is set to `true`, the data written will be
    * flushed to the file system before the returned future completes.
    *
    */
   Future<File> writeAsString(String contents,
-      {FileMode mode: FileMode.WRITE,
-      Encoding encoding: UTF8,
+      {FileMode mode: FileMode.write,
+      Encoding encoding: utf8,
       bool flush: false});
 
   /**
@@ -563,7 +608,7 @@ abstract class File implements FileSystemEntity {
    *
    * By default [writeAsStringSync] creates the file for writing and
    * truncates the file if it already exists. In order to append the bytes
-   * to an existing file, pass [FileMode.APPEND] as the optional mode
+   * to an existing file, pass [FileMode.append] as the optional mode
    * parameter.
    *
    * If the [flush] argument is set to `true` data written will be
@@ -572,8 +617,8 @@ abstract class File implements FileSystemEntity {
    * Throws a [FileSystemException] if the operation fails.
    */
   void writeAsStringSync(String contents,
-      {FileMode mode: FileMode.WRITE,
-      Encoding encoding: UTF8,
+      {FileMode mode: FileMode.write,
+      Encoding encoding: utf8,
       bool flush: false});
 
   /**
@@ -603,10 +648,10 @@ abstract class File implements FileSystemEntity {
  */
 abstract class RandomAccessFile {
   /**
-   * Closes the file. Returns a `Future<RandomAccessFile>` that
-   * completes with this RandomAccessFile when it has been closed.
+   * Closes the file. Returns a `Future` that
+   * completes when it has been closed.
    */
-  Future<RandomAccessFile> close();
+  Future<void> close();
 
   /**
    * Synchronously closes the file.
@@ -632,7 +677,7 @@ abstract class RandomAccessFile {
   /**
    * Reads [bytes] bytes from a file and returns the result as a list of bytes.
    */
-  Future<List<int>> read(int bytes);
+  Future<Uint8List> read(int bytes);
 
   /**
    * Synchronously reads a maximum of [bytes] bytes from a file and
@@ -640,7 +685,7 @@ abstract class RandomAccessFile {
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  List<int> readSync(int bytes);
+  Uint8List readSync(int bytes);
 
   /**
    * Reads into an existing [List<int>] from the file. If [start] is present,
@@ -651,18 +696,20 @@ abstract class RandomAccessFile {
    *
    * Returns a `Future<int>` that completes with the number of bytes read.
    */
-  Future<int> readInto(List<int> buffer, [int start = 0, int end]);
+  Future<int> readInto(List<int> buffer, [int start = 0, int? end]);
 
   /**
-   * Synchronously reads into an existing [List<int>] from the file. If [start]
-   * is present, the bytes will be filled into [buffer] from at index [start],
-   * otherwise index 0.  If [end] is present, the [end] - [start] bytes will be
-   * read into [buffer], otherwise up to [buffer.length]. If [end] == [start]
-   * nothing happens.
+   * Synchronously reads into an existing [List<int>] from the file and returns
+   * the number of bytes read.
+   *
+   * If [start] is present, the bytes will be filled into [buffer] from at
+   * index [start], otherwise index 0.  If [end] is present, the
+   * [end] - [start] bytes will be read into [buffer], otherwise up to
+   * [buffer.length]. If [end] == [start] nothing happens.
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  int readIntoSync(List<int> buffer, [int start = 0, int end]);
+  int readIntoSync(List<int> buffer, [int start = 0, int? end]);
 
   /**
    * Writes a single byte to the file. Returns a
@@ -672,8 +719,7 @@ abstract class RandomAccessFile {
   Future<RandomAccessFile> writeByte(int value);
 
   /**
-   * Synchronously writes a single byte to the file. Returns the
-   * number of bytes successfully written.
+   * Synchronously writes a single byte to the file. Returns 1 on success.
    *
    * Throws a [FileSystemException] if the operation fails.
    */
@@ -688,7 +734,7 @@ abstract class RandomAccessFile {
    * [RandomAccessFile] when the write completes.
    */
   Future<RandomAccessFile> writeFrom(List<int> buffer,
-      [int start = 0, int end]);
+      [int start = 0, int? end]);
 
   /**
    * Synchronously writes from a [List<int>] to the file. It will read the
@@ -698,7 +744,7 @@ abstract class RandomAccessFile {
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  void writeFromSync(List<int> buffer, [int start = 0, int end]);
+  void writeFromSync(List<int> buffer, [int start = 0, int? end]);
 
   /**
    * Writes a string to the file using the given [Encoding]. Returns a
@@ -706,7 +752,7 @@ abstract class RandomAccessFile {
    * RandomAccessFile when the write completes.
    */
   Future<RandomAccessFile> writeString(String string,
-      {Encoding encoding: UTF8});
+      {Encoding encoding: utf8});
 
   /**
    * Synchronously writes a single string to the file using the given
@@ -714,7 +760,7 @@ abstract class RandomAccessFile {
    *
    * Throws a [FileSystemException] if the operation fails.
    */
-  void writeStringSync(String string, {Encoding encoding: UTF8});
+  void writeStringSync(String string, {Encoding encoding: utf8});
 
   /**
    * Gets the current byte position in the file. Returns a
@@ -799,9 +845,9 @@ abstract class RandomAccessFile {
    *
    * To obtain an exclusive lock on a file it must be opened for writing.
    *
-   * If [mode] is [FileLock.EXCLUSIVE] or [FileLock.SHARED], an error is
+   * If [mode] is [FileLock.exclusive] or [FileLock.shared], an error is
    * signaled if the lock cannot be obtained. If [mode] is
-   * [FileLock.BLOCKING_EXCLUSIVE] or [FileLock.BLOCKING_SHARED], the
+   * [FileLock.blockingExclusive] or [FileLock.blockingShared], the
    * returned [Future] is resolved only when the lock has been obtained.
    *
    * *NOTE* file locking does have slight differences in behavior across
@@ -820,7 +866,7 @@ abstract class RandomAccessFile {
    * already unlocked".
    */
   Future<RandomAccessFile> lock(
-      [FileLock mode = FileLock.EXCLUSIVE, int start = 0, int end = -1]);
+      [FileLock mode = FileLock.exclusive, int start = 0, int end = -1]);
 
   /**
    * Synchronously locks the file or part of the file.
@@ -837,9 +883,9 @@ abstract class RandomAccessFile {
    *
    * To obtain an exclusive lock on a file it must be opened for writing.
    *
-   * If [mode] is [FileLock.EXCLUSIVE] or [FileLock.SHARED], an exception is
+   * If [mode] is [FileLock.exclusive] or [FileLock.shared], an exception is
    * thrown if the lock cannot be obtained. If [mode] is
-   * [FileLock.BLOCKING_EXCLUSIVE] or [FileLock.BLOCKING_SHARED], the
+   * [FileLock.blockingExclusive] or [FileLock.blockingShared], the
    * call returns only after the lock has been obtained.
    *
    * *NOTE* file locking does have slight differences in behavior across
@@ -859,7 +905,7 @@ abstract class RandomAccessFile {
    *
    */
   void lockSync(
-      [FileLock mode = FileLock.EXCLUSIVE, int start = 0, int end = -1]);
+      [FileLock mode = FileLock.exclusive, int start = 0, int end = -1]);
 
   /**
    * Unlocks the file or part of the file.
@@ -907,6 +953,7 @@ abstract class RandomAccessFile {
 /**
  * Exception thrown when a file operation fails.
  */
+@pragma("vm:entry-point")
 class FileSystemException implements IOException {
   /**
    * Message describing the error. This does not include any detailed
@@ -919,25 +966,26 @@ class FileSystemException implements IOException {
    * The file system path on which the error occurred. Can be `null`
    * if the exception does not relate directly to a file system path.
    */
-  final String path;
+  final String? path;
 
   /**
    * The underlying OS error. Can be `null` if the exception is not
    * raised due to an OS error.
    */
-  final OSError osError;
+  final OSError? osError;
 
   /**
    * Creates a new FileSystemException with an optional error message
    * [message], optional file system path [path] and optional OS error
    * [osError].
    */
+  @pragma("vm:entry-point")
   const FileSystemException([this.message = "", this.path = "", this.osError]);
 
   String toString() {
     StringBuffer sb = new StringBuffer();
     sb.write("FileSystemException");
-    if (!message.isEmpty) {
+    if (message.isNotEmpty) {
       sb.write(": $message");
       if (path != null) {
         sb.write(", path = '$path'");
